unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons,
  DSound,
  afDXSPlay, afDXSUtil;

type
  TfrmMain = class(TForm)
    MemoStatus: TMemo;
    btnSelectWave: TButton;
    btnPlay: TButton;
    btnStop: TButton;
    btnClose: TButton;
    OpenDialog1: TOpenDialog;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnSelectWaveClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private
    { Private declarations }
    WaveFile : TafWAVEStream;
    WaveDone : Boolean;
    Buff : TafDXSPlayStreamingBuffer;
    procedure DataRequired(Sender : TObject);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Set up the Playback object
  DXSPlay_Init(Self, Handle, DSSCL_PRIORITY);
  if Assigned(DXSPlay) and DXSPlay.Ready then MemoStatus.Lines.Add('DX Sound Playback available.')
    else MemoStatus.Lines.Add('DX Sound Playback not available.');

  // initialize the other objects we require (file and buffer)
  WaveFile := nil;
  Buff := nil;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  btnClose.Click;  // be sure file and buffer objects are freed
  DXSPlay_Free;    // Free the Playback object
end;

procedure TfrmMain.DataRequired(Sender : TObject);
var
  Ptr : Pointer;
  Size, SizeRead : Longint;
begin
  // This is called when the blayback buffer needs data...
  // It will be called immediately on play, and any data you insert
  // cannot start to be heard until the buffer loops around again
  // this means there's always a startup delay (because the streaming
  // buffer cannot be "pre-loaded")

  if (WaveDone) and (Buff.QueueCount = 0) then btnStop.Click;
  // wave has been read and buffer played completely

  Ptr := nil;
  Size := 10000;     // note this must be half of the buffer's size
                     // buffer data inserts are in half buffer increments
  GetMem(Ptr, Size); // allocating memory for the wave data from the file

  if Assigned(WaveFile) then begin
    // read next set of wave data from the file
    SizeRead := WaveFile.Read(Ptr^, Size);
    // note the pointer dereference above... this is necessary when
    // reading to a buffer specified by a pointer.
    if SizeRead > 0 then begin
      // if we read any data, insert it into the streaming buffer
      Buff.InsertData(Ptr, Size);
      MemoStatus.Lines.Add('Reading from file');
    end else begin
      // if we didn't read any data, it's because the file's finished
      WaveDone := True;
      // we set a global variable to indicate this (file read) so the next time
      // the buffer needs data, we'll know there can be no more to supply
      // and we'll stop the buffer then. (because right now, there's still
      // data in the buffer that must be played out)
    end;
  end;
  FreeMem(Ptr, Size);
  // free the memory we used to hold the data from the file
end;

procedure TfrmMain.btnSelectWaveClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    if Assigned(DXSPlay) then begin
      // if the DX Sound Object was created
      if Assigned(WaveFile) then WaveFile.Free;
      // if a wave file is open, close it
      WaveFile := TafWAVEStream.Create(OpenDialog1.Filename,
                                       fmOpenRead + fmShareDenyWrite);
      // open the wave file for reading (takes same args as TFileStream)
      if Assigned(Buff) then Buff.Free;
      // if the buffer is open, then free it
      Buff := DXSPlay.NewStreamingBuffer(WaveFile.Channels,
                                         WaveFile.Bits,
                                         WaveFile.SamplesPerSecond,
                                         0,
                                         20000);
      // create a buffer to play the wave (note buffer total size is 20000)
      // the streaming buffer size must ALWAYS be divisible by 2 !!!!
      Buff.OnDataRequired := DataRequired;
      // Tell the buffer where to request data to play
      MemoStatus.Lines.Add('Playback Buffer Created.');
      MemoStatus.Lines.Add('Wave File (' + OpenDialog1.Filename + ') Opened.');
    end else begin
      MemoStatus.Lines.Add('Playback not enabled.');
    end;
end;

procedure TfrmMain.btnPlayClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the playback buffer exists, then...
    if Assigned(WaveFile) then WaveFile.Position := 0;
    // set the wave file position to the start of the file
    WaveDone := False;   // indicate wave file is not finished
    Buff.PlayLoop;       // start playback
    MemoStatus.Lines.Add('Playback Started.');
  end;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the playback buffer exists, then...
    Buff.Stop;        // stop the buffer playback
    Buff.ClearQueue;  // clear the buffer of any remaining data (not yet played)
    MemoStatus.Lines.Add('Playback Stopped.');
    // set the wave file position to the start of the file
    if Assigned(WaveFile) then WaveFile.Position := 0;
    WaveDone := False;   // indicate wave file is not finished
    MemoStatus.Lines.Add('Wave File reset to Start.');
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the playback buffer exists, then...
    Buff.Free;    // free the buffer
    Buff := nil;  // reset its pointer to nil so we know it's been freed!
    MemoStatus.Lines.Add('Playback Buffer Destroyed.');
  end;
  if Assigned(WaveFile) then begin
    // if the wave file exists, then...
    WaveFile.Free;    // free the wave file
    WaveFile := nil;  // reset its pointer to nil so we know it's been freed!
    MemoStatus.Lines.Add('Wave File Closed.');
  end;
end;


end.
